@@ -1,6 +1,9 @@
+import errno
+import math
 import os.path
 import re
 import shlex
+import stat
 import string
 import sys
 from typing import IO
@@ -18,13 +21,14 @@ printable = frozenset(string.printable)
 
 DIRECTORY = 'directory'
 SYMLINK = 'symlink'
+SOCKET = 'socket'
 FILE = 'file'
 EXECUTABLE = 'executable'
 NON_EXECUTABLE = 'non-executable'
 TEXT = 'text'
 BINARY = 'binary'
 
-TYPE_TAGS = frozenset((DIRECTORY, FILE, SYMLINK))
+TYPE_TAGS = frozenset((DIRECTORY, FILE, SYMLINK, SOCKET))
 MODE_TAGS = frozenset((EXECUTABLE, NON_EXECUTABLE))
 ENCODING_TAGS = frozenset((BINARY, TEXT))
 _ALL_TAGS = {*TYPE_TAGS, *MODE_TAGS, *ENCODING_TAGS}
@@ -36,12 +40,18 @@ ALL_TAGS = frozenset(_ALL_TAGS)
 
 
 def tags_from_path(path: str) -> Set[str]:
-    if not os.path.lexists(path):
+    try:
+        sr = os.lstat(path)
+    except (OSError, ValueError):  # same error-handling as `os.lexists()`
         raise ValueError(f'{path} does not exist.')
-    if os.path.isdir(path):
+
+    mode = sr.st_mode
+    if stat.S_ISDIR(mode):
         return {DIRECTORY}
-    if os.path.islink(path):
+    if stat.S_ISLNK(mode):
         return {SYMLINK}
+    if stat.S_ISSOCK(mode):
+        return {SOCKET}
 
     tags = {FILE}
 
@@ -184,7 +194,11 @@ def parse_shebang(bytesio: IO[bytes]) -> Tuple[str, ...]:
 
     cmd = tuple(_shebang_split(first_line.strip()))
     if cmd and cmd[0] == '/usr/bin/env':
-        cmd = cmd[1:]
+        if cmd[1] == '-S':
+            cmd = cmd[2:]
+        else:
+            cmd = cmd[1:]
+
         if cmd == ('nix-shell',):
             return _parse_nix_shebang(bytesio, cmd)
     return cmd
@@ -197,8 +211,14 @@ def parse_shebang_from_file(path: str) -> Tuple[str, ...]:
     if not os.access(path, os.X_OK):
         return ()
 
-    with open(path, 'rb') as f:
-        return parse_shebang(f)
+    try:
+        with open(path, 'rb') as f:
+            return parse_shebang(f)
+    except OSError as e:
+        if e.errno == errno.EINVAL:
+            return ()
+        else:
+            raise
 
 
 COPYRIGHT_RE = re.compile(r'^\s*(Copyright|\(C\)) .*$', re.I | re.MULTILINE)
@@ -225,7 +245,7 @@ def license_id(filename: str) -> Optional[str]:
     3. check exact text match with existing licenses
     4. failing that use edit distance
     """
-    import editdistance  # `pip install identify[license]`
+    import ukkonen  # `pip install identify[license]`
 
     with open(filename, encoding='UTF-8') as f:
         contents = f.read()
@@ -235,6 +255,8 @@ def license_id(filename: str) -> Optional[str]:
     min_edit_dist = sys.maxsize
     min_edit_dist_spdx = ''
 
+    cutoff = math.ceil(.05 * len(norm))
+
     # try exact matches
     for spdx, text in licenses.LICENSES:
         norm_license = _norm_license(text)
@@ -245,13 +267,13 @@ def license_id(filename: str) -> Optional[str]:
         if norm and abs(len(norm) - len(norm_license)) / len(norm) > .05:
             continue
 
-        edit_dist = editdistance.eval(norm, norm_license)
-        if edit_dist < min_edit_dist:
+        edit_dist = ukkonen.distance(norm, norm_license, cutoff)
+        if edit_dist < cutoff and edit_dist < min_edit_dist:
             min_edit_dist = edit_dist
             min_edit_dist_spdx = spdx
 
     # if there's less than 5% edited from the license, we found our match
-    if norm and min_edit_dist / len(norm) < .05:
+    if norm and min_edit_dist < cutoff:
         return min_edit_dist_spdx
     else:
         # no matches :'(
